######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

#  BRUGS program for hierarchical modeling of successive
#  removal data with habitat and time covariates
#  Application to the Oir River data set
#  Chapter 10 - Section 10.1

######################################################################################
######################################################################################


rm(list=ls())

library(BRugs)
library(coda)

# Load model / data / inits
# -----------------------------------------------------------

model <- "model_Hierarchical_SuccRem1_Oir.txt"
data <- "data_Hierarchical_SuccRem1_Oir.txt"

inits1 <- "inits1_Hierarchical_SuccRem1_Oir.txt"
inits2 <- "inits2_Hierarchical_SuccRem1_Oir.txt"
inits3 <- "inits3_Hierarchical_SuccRem1_Oir.txt"

# Data values

data.values <- source(data)$value
smolts <- data.values$smolts
n <- data.values$n
ny <- data.values$ny

# Year 1990 is missing in the data set (No Electrofishing data for year 1990)
# Number of smolts for this year has been deleted

years <- c(1985:1989,1991:2005)
years.labels <- as.character(years)



# MCMC config.
# -----------------------------------------------------------

n.burnin <- 10000
n.update <- 10000
n.thin <- 10
n.chains <- 3



# Run OpenBUGS                            
# ------------------------------------------------------------

modelCheck(model)  
modelData(data)   
modelCompile(numChains=n.chains)    

modelInits(c(inits1, inits2, inits3))  
modelGenInits()

# Burn-in period

ptm<-proc.time()
modelUpdate(numUpdates = n.burnin, thin = n.thin) 
proc.time()-ptm

# Inferences

# Parameters to estimate
monitor <- c(
"alpha_d", "beta_d","mu_p","vp","mu_d","vd","v_ad",
"correlation",
"N_samp_i","Ntot","surv",
"T1reptot","T1obstot","T2reptot","T2obstot", "ind.T1", "ind.T2")

ptm <- proc.time()
samplesSet(monitor)
modelUpdate(numUpdates = n.update, thin = n.thin)
proc.time()-ptm



# Save results                            
# ------------------------------------------------------------

mu_p <- as.matrix(buildMCMC("mu_p"))[,1]
mu_d <- as.matrix(buildMCMC("mu_d"))[,1]
vd <- as.matrix(buildMCMC("vd"))[,1]
correlation <- as.matrix(buildMCMC("correlation"))[,1]
alpha_d <- as.matrix(buildMCMC("alpha_d"))
beta_d <- as.matrix(buildMCMC("beta_d"))

N_samp_i <- as.matrix(buildMCMC("N_samp_i"))
Ntot <- as.matrix(buildMCMC("Ntot"))
 
surv <- as.matrix(buildMCMC("surv"))

T1obs <- as.matrix(buildMCMC("T1obstot"))[,1]
T1rep <- as.matrix(buildMCMC("T1reptot"))[,1]
T2obs <- as.matrix(buildMCMC("T2obstot"))[,1]
T2rep <- as.matrix(buildMCMC("T2reptot"))[,1]
indT1 <- as.matrix(buildMCMC("ind.T1"))[,1]
indT2 <- as.matrix(buildMCMC("ind.T2"))[,1]



# Stats                            
# ------------------------------------------------------------

samplesStats("*")



# Extrapolation
# ----------------------------------------------------------------

# Data for extrapolation
# ----------------------

nsim = length(mu_d)
ny <- win.data$ny
S=win.data$S
n_sites_ex = t(structure(.Data=c(
29, 76,
29, 76,
49, 159,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 183,
50, 165,
50,  183,
50, 165, 
45, 165,
50, 183,
50, 183,
50, 183
),.Dim=c(2,20)))
s_ex=t(structure(.Data=c(
4.20, 3.43,
4.20, 3.43,
2.62, 1.64,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43,
2.57, 1.59,
2.57, 1.43,
2.57, 1.59, 
2.89, 1.59,
2.57, 1.43,
2.57, 1.43,
2.57, 1.43
),.Dim=c(2,20)))


# Calculate Ntot with no parameters uncertainty (only spatial variability)
# ------------------------------------------------------------------------

logEih_ex<-array(0,c(nsim,ny,2))
N_ex_ihparfait<-array(0,c(nsim,ny,2))
for (i in 1:ny) { 
for (h in 1:2) {
logEih_ex[,i,h] <- mu_d + alpha_d[,i] + beta_d[,h]

#  Extrapolation using n_sites_ex

d_ex_2parfait<-array(0,c(nsim,n_sites_ex[i,h]))
N_ex_2parfait<-array(0,c(nsim,n_sites_ex[i,h]))
for (k in 1:n_sites_ex[i,h])
{
d_ex_2parfait[,k] <- exp(rnorm(nsim,mean(logEih_ex[,i,h]),mean(vd^0.5)))
N_ex_2parfait[,k] <-rpois(nsim,d_ex_2parfait[,k] * s_ex[i,h])
}  # end loop k

N_ex_ihparfait[,i,h] <- apply(N_ex_2parfait,1, sum)

}  # end loop h
}  # end loop i

Ntotparfait <- 0*Ntot
N_samp_mean <- apply(N_samp_i,2,mean)
for (sim in 1:nsim)
{
Ntotparfait[sim,]=N_samp_mean+N_ex_ihparfait[sim,,1]+N_ex_ihparfait[sim,,2]
}




# Figures                            
# ------------------------------------------------------------

size.labels = 1.0
size.text = 1.3
col1 <- "grey90"
col2 <- "grey55"
col1 = c(rep(col1, times=n))
col2 = c(rep(col2, times=n))
box.size1 = 0.4
box.size2 = 0.6


# alpha(year)
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Fig6_alpha_d.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

X1 <- alpha_d
labels <- years.labels

x.min = 1 ; x.max = ny+1
y.min = -4 ; y.max = 3
y.label = expression(alpha[delta])
x.label = "Years"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:ny) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

axis(side =1, at=1:ny, labels = labels, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3.5, side = 1, cex=size.text)
mtext(y.label, line = 3, side = 2, cex=size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()


# beta(habitat)
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Fig6_beta_d.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

X1 <- beta_d
n <- 2
labels <- c("Rapids","Runs")

x.min = 0.5 ; x.max = n+0.5
y.min = -1.5 ; y.max = 1.5
y.label = expression(beta[delta])
x.label = "Habitat type"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

axis(side =1, at=1:n, labels = labels, las=1, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels, pos = 0.7)
mtext(x.label, line = 3, side = 1, cex=size.text)
mtext(y.label, line = 1, side = 2, cex=size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()



# Time series Ntot
# ---------------------------------------------

N1 <- Ntot
N2 <- Ntotparfait
labels <- years.labels

x.label <- "Years" 
y.label <- "Number of 0+ juveniles"

# windows()

res <- 6
name_figure <- "Fig6_PredictionNtot.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(bty="n", mar=c(6,5,1,1))
matplot(	1:ny,t(apply(N1,2,quantile,p=c(0.1,0.5,0.9))),
		t='l', col=c(1,2,1), lty=c(3,1,3), lwd=c(2,2,2),
 		xlab="", ylab="", xaxt = "n", yaxt = "n",)

xx <- c(1:ny,rev(1:ny))
yy <- c(apply(N2,2,quantile,0.9),rev(apply(N2,2,quantile,0.1)))
polygon(xx,yy,col="lightgray", border = NA, xaxt = "n", yaxt = "n")

lines(1:ny,apply(N1,2,median),lwd=3, lty=1)

axis(side = 1, at = 1:ny, labels = labels, las = 3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)

mtext(x.label, line = 3.5, side = 1, cex=size.text)
mtext(y.label, line = 3, side = 2, cex=size.text)
# mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()


# Coeficient of correlation
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Fig6_Correlation.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(bty="n", mar=c(6,5,1,1))

# x.label <- "Correlation between fish density and capture prob."
x.label = expression(rho[delta][pi])

hist(correlation,50,main="",xlab="",ylab="",freq=F, cex = size.text, xaxt="n", yaxt="n")
lines(density(correlation), col="black", lwd=2)

axis(side =1, cex.axis=size.labels)
axis(side =2, cex.axis = size.labels)

mtext(x.label, line = 3, side = 1, cex=size.text+0.5)
mtext("", line = 3, side = 2, cex=size.text)
# mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()


# Checking with smolts number
# ----------------------------------------------

juv <- apply(Ntot,2,mean)[2:18]
juv10 <- apply(Ntot,2,quantile,0.1)[2:18]
juv90 <- apply(Ntot,2,quantile,0.9)[2:18]

smolts2 <- smolts[2:18]

x.label <- "0+ Juveniles"
y.label <- "Smolts (points estimates)"

# windows()

res <- 6
name_figure <- "Fig6_JuvSmolts.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(bty="n", mar=c(6,5,1,1))

plot(juv,smolts2, xlab="", ylab="", xlim = c(0,7000), ylim=c(0,3000), xaxt="n", yaxt = "n", pch=19)
lines(0:3000,0:3000,lwd=3,lty=1)
for (t in 1:18){lines(c((juv10[t]),(juv90[t])),c(smolts2[t],smolts2[t]))}

text(juv,smolts2,years[2:18],cex=0.8, pos=c(3,3,3,3,1,3,3,3,3,3,1,1,3,3,3,3,3,3,3,3))

abline(a=0,b=exp(mean(log(smolts2)-log(juv))), lwd=1, lty=2)

axis(side =1, las=1, cex.axis=size.labels)
axis(side =2, las=1, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex=size.text)
mtext(y.label, line = 3.5, side = 2, cex=size.text)
# mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()


# Posterior checking
# ----------------------------------------------

pval1 <- mean(indT1)
pval2 <- mean(indT2)
pval1
pval2

# windows()

res <- 6
name_figure <- "Fig6_OmnibusChisq.png"
png(filename = name_figure, height = 500*res, width = 1000*res, res=72*res)

par(bty="n", mar=c(6,6,1,1), mfrow=c(1,2))

# C1

plot(T1obs[T1obs<max(T1rep)],T1rep[T1obs<max(T1rep)],t="p", cex=2, pch='.',
	xlab="", ylab="", xaxt="n", yaxt="n",
	xlim=c(100,250), ylim=c(50,250))

abline(a=0,b=1, col="black", lty=1, lwd=2)

x.label <- expression(paste({chi^{2}}[(1)](C^1,pi,nu)))
y.label <- expression(paste({chi^{2}}[(1)]({C^1}^{rep},pi,nu)))

axis(side =1, las=1, cex.axis=size.labels)
axis(side =2, las=1, cex.axis=size.labels)

mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3.5, side = 2, cex = size.text)
mtext("a)",side=3,adj=0, cex=size.text)


# C2

plot(T2obs[T2obs<max(T2rep)],T2rep[T2obs<max(T2rep)],t="p", cex=2,pch=".",
	xlab="", ylab="", xaxt="n", yaxt="n",
	xlim=c(0,150),ylim=c(0,150))

abline(a=0,b=1, col="black", lty=1, lwd=2)

x.label =expression(paste({chi^{2}}[(2)]({C^1},{C^2},pi,nu)))
y.label =expression(paste({chi^{2}}[(2)]({C^1}^rep,{C^2}^rep,pi,nu)))

axis(side =1, las=1, cex.axis=size.labels)
axis(side =2, las=1, cex.axis=size.labels)

mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3.5, side = 2, cex = size.text)
mtext("b)",side=3,adj=0, cex=size.text)

dev.off()


# Smolts survival
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Fig6_SurvivalRates.png"
png(filename = name_figure, height = 500*res, width = 750*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

n <- ny-2
X1 <- surv[,1:n]
labels <- as.character(years)[1:n]

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 1
y.label = "Survival 0+ - Smolts"
x.label = "Years"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size1, col = col2[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

axis(side =1, at=1:n, labels = labels, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex=size.text)
mtext(y.label, line = 3, side = 2, cex=size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()

